@@ -666,7 +666,7 @@ pca.mvreplace <- function(x, center = TRUE, scale = FALSE, maxncomp = 10, expvar
    }
 
    if (center) {
-      gmean <- attr(xo, "scaled:center");
+      gmean <- attr(xo, "scaled:center")
    }
 
    n <- 1
@@ -697,7 +697,7 @@ pca.mvreplace <- function(x, center = TRUE, scale = FALSE, maxncomp = 10, expvar
       x_new <- tcrossprod(scores, loadings)
 
       # remove centering
-      x_new <- sweep(x_new, 2L, lmean, "+", check.margin = F)
+      x_new <- sweep(x_new, 2L, lmean, "+", check.margin = FALSE)
 
       # replace missing values by the calculated
       x <- xo
@@ -831,7 +831,7 @@ pca.nipals <- function(x, ncomp = min(ncol(x), nrow(x) - 1), tol = 10^-10) {
    E <- x
    for (i in seq_len(ncomp)) {
       ind <- which.max(apply(E, 2, sd))
-      t <- E[, ind, drop = F]
+      t <- E[, ind, drop = FALSE]
       tau <- th <- 99999999
       while (th > tol * tau) {
          p <- crossprod(E, t) / as.vector(crossprod(t))
@@ -1091,9 +1091,13 @@ plotCumVariance.pca <- function(obj, legend.position = "bottomright", ...) {
 #' See examples in help for \code{\link{pca}} function.
 #'
 #' @export
-plotScores.pca <- function(obj, comp = c(1, 2), type = "p", show.axes = TRUE,
+plotScores.pca <- function(obj, comp = if (obj$ncomp > 1) c(1, 2) else 1, type = "p", show.axes = TRUE,
    show.legend = TRUE, res = obj$res, ...) {
 
+   if (min(comp) < 1 || max(comp) > ncol(obj$loadings)) {
+      stop("Wrong values for 'comp' parameter.")
+   }
+
    res <- getRes(res, "ldecomp")
    if (length(res) == 1) {
       return(plotScores(res[[1]], comp = comp, type = type, ...))
@@ -1207,9 +1211,14 @@ plotResiduals.pca <- function(obj, ncomp = obj$ncomp.selected, log = FALSE,
 #' See examples in help for \code{\link{pca}} function.
 #'
 #' @export
-plotLoadings.pca <- function(obj, comp = c(1, 2), type = (if (length(comp == 2)) "p" else "l"),
+plotLoadings.pca <- function(obj, comp = if (obj$ncomp > 1) c(1, 2) else 1,
+   type = (if (length(comp == 2)) "p" else "l"),
    show.legend = TRUE, show.axes = TRUE, ...) {
 
+   if (min(comp) < 1 || max(comp) > ncol(obj$loadings)) {
+      stop("Wrong values for 'comp' parameter.")
+   }
+
    plot_data <- mda.subset(obj$loadings, select = comp)
    colnames(plot_data) <- paste0("Comp ", comp, " (", round(obj$res[["cal"]]$expvar[comp], 2), "%)")
    attr(plot_data, "name") <- "Loadings"
@@ -1284,7 +1293,7 @@ plotBiplot.pca <- function(obj, comp = c(1, 2), pch = c(16, NA), col = mdaplot.g
             main = main, colmap = col, show.lines = show.lines, show.excluded = show.excluded, ...)
 
    if (show.excluded && length(attr(loadings, "exclrows")) > 0) {
-      loadings <- loadings[-attr(loadings, "exclrows"), , drop = F]
+      loadings <- loadings[-attr(loadings, "exclrows"), , drop = FALSE]
    }
 
    segments(0, 0, loadings[, 1], loadings[, 2], col = col[2], lty = lty, lwd = lwd)
